@@ -28,9 +28,6 @@
  #include <unistd.h>
  #include <langinfo.h>
  #include <limits.h>
- #if defined(IRIX)
-  #define PATH_MAX FILENAME_MAX
- #endif
 #endif
 
 #include "wrapper_file.h"
