@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2017 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -87,6 +87,25 @@ extern void hashMapPutKWVW(PHashMap hashMap, const TCHAR *key, const TCHAR *valu
  */
 extern void hashMapPutKMBVW(PHashMap hashMap, const char *key, const TCHAR *value);
 
+/**
+ * Puts an integer value into the HashMap.
+ *  Note: Avoid putting 0 in the hashtable as hashMapGetKWVI() also returns 0 if the key was not found. 
+ *
+ * @param hashMap HashMap to store the value into.
+ * @param key The key to reference the value.
+ * @param value The value to store.
+ */
+extern void hashMapPutKWVI(PHashMap hashMap, const TCHAR *key, int value);
+
+/**
+ * Puts a value into the HashMap.  The key and value will both be cloned.
+ *
+ * @param hashMap HashMap to store the value into.
+ * @param key The key to reference the value.
+ * @param value The value to store.
+ */
+extern void hashMapPutKIVW(PHashMap hashMap, int key, const TCHAR *value);
+
 /**
  * Gets a value from the HashMap.
  *
@@ -119,6 +138,26 @@ extern const TCHAR *hashMapGetKWVW(PHashMap hashMap, const TCHAR *key);
  */
 extern const TCHAR *hashMapGetKMBVW(PHashMap hashMap, const char *key);
 
+/**
+ * Gets an integer value from the HashMap.
+ *
+ * @param hashMap HashMap from which to lookup the value.
+ * @param key Key of the value being looked up.
+ *
+ * @return the integer value or 0 if the key was not found.
+ */
+extern int hashMapGetKWVI(PHashMap hashMap, const TCHAR *key);
+
+/**
+ * Gets an integer value from the HashMap.
+ *
+ * @param hashMap HashMap from which to lookup the value.
+ * @param key Key of the value being looked up.
+ *
+ * @return the integer value or 0 if the key was not found.
+ */
+extern const TCHAR *hashMapGetKIVW(PHashMap hashMap, int key);
+
  #ifdef _DEBUG_HASHMAP
 /**
  * Dumps the statistics of a HashMap.
